@@ -1020,10 +1020,12 @@ class RandAugment(ImageAugment):
     def select_and_apply_random_policy(self, image):
         """Select a random policy from `policies` and apply it to `image`."""
         policy_to_select = tf.random.uniform([], maxval=len(self.available_ops), dtype=tf.int32)
+        branch_fns = [(id, lambda selected=ii: self.apply_policy(selected, image)) for id, ii in enumerate(self.available_ops)]
+        image = tf.switch_case(branch_index=policy_to_select, branch_fns=branch_fns, default=lambda: tf.identity(image))
         # Note that using tf.case instead of tf.conds would result in significantly
         # larger graphs and would even break export for some larger policies.
-        for (i, policy) in enumerate(self.available_ops):
-            image = tf.cond(tf.equal(i, policy_to_select), lambda policy=policy: self.apply_policy(policy, image), lambda: image)
+        # for (i, policy) in enumerate(self.available_ops):
+        #     image = tf.cond(tf.equal(i, policy_to_select), lambda policy=policy: self.apply_policy(policy, image), lambda: image)
         return image
 
     def __call__(self, image: tf.Tensor) -> tf.Tensor:
